unit MainDlg;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Mask, ExtCtrls, ComCtrls, IniFiles;

type
  TMainForm = class(TForm)
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    TabSheet3: TTabSheet;
    TabSheet4: TTabSheet;
    Bevel3: TBevel;
    Label2: TLabel;
    PartitaIva: TMaskEdit;
    OutPiva: TStaticText;
    PItestBtn: TButton;
    Bevel2: TBevel;
    Label1: TLabel;
    CodiceFiscale: TMaskEdit;
    OutCodFis: TStaticText;
    CFtestBtn: TButton;
    Label3: TLabel;
    InfoLbl: TLabel;
    VersionLbl: TLabel;
    Label4: TLabel;
    Image1: TImage;
    Panel2: TPanel;
    CloseBtn: TButton;
    Memo1: TMemo;
    Bevel1: TBevel;
    Edit_Rate: TEdit;
    Edit_Lira: TEdit;
    Edit_Euro: TEdit;
    CalcLira: TStaticText;
    CalcEuro: TStaticText;
    EuroFlag: TCheckBox;
    ExecBtn: TButton;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    function  CheckCodiceFiscale(CF: string): boolean;
    function  CheckPartitaIva(PI: string): boolean;
    procedure ReadIniFile;
    procedure EuroCalc;
    { -o- }
    procedure CFtestBtnClick(Sender: TObject);
    procedure PItestBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure CloseBtnClick(Sender: TObject);
    procedure ExecBtnClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;
  Rate,
  Euro,
  Lira:  Currency;

implementation

{$R *.DFM}

{-------------------------------------------------------------------------------
}
procedure TMainForm.FormCreate(Sender: TObject);
begin
  VersionLbl.Caption := 'v. 1.1b - 27.12.1998';
  InfoLbl.Caption    := 'This is FreeWare';
  PageControl1.ActivePage.PageIndex := 0;
  ReadIniFile;
end;



{-------------------------------------------------------------------------------
  small function to strip all blanks in a string
}
function StripAllBlanks(S : string): string;
var I,L: Integer;
begin
  L := Length(S);
  I := 1;
  repeat
	   if (S[I] = ' ') then begin
       Delete(S, I, 1);
       Dec(L);
    end;
    Inc(I);
  until I >= L;
	 Result := S;
end; {- StripAllBlanks }



{-------------------------------------------------------------------------------
  Checking if the italian personal fiscal code is correct.
  The last digit is the checksum, so we scan the first 15 digits and comparing
  with the tables in const section.
  The function is stand-alone so you can use with no problems
}
function TMainForm.CheckCodiceFiscale(CF: string): boolean;
const Alpha1 = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
      Digit1 = '0123456789';
      Alpha2 = 'BAKPLCQDREVOSFTGUHMINJWZYX';
      Digit2 = '10   2 3 4   5 6 7 8 9    ';
var   Ch, Code: char;
      Check, It: integer;
begin

  Check := 0;  It := 2;
  repeat {- even loop }
    Ch := CF[It];
    Case Ch of
      '0'..'9': Check := Check + Pos(Ch, Digit1);
      'A'..'Z': Check := Check + Pos(Ch, Alpha1);
    end;
    It := It + 2;
  until It > 14;

  It := 1;
  repeat {- odd loop }
    Ch := CF[It];
    Case Ch of
      '0'..'9': Check := Check + Pos(Ch, Digit2);
      'A'..'Z': Check := Check + Pos(Ch, Alpha2);
    end;
    It := It + 2;
  until It > 15;

  Check := 1 + ((Check - 15) mod 26) + 64;       {- calculate the Check digit }
  Code := Chr(Check);                            {- translate in char }
  Ch := CF[16];                                  {- get last digit entered }
  if Code = Ch then                              {- then compare }
      CheckCodiceFiscale := True
  else CheckCodiceFiscale := False;
end; {- CheckCodiceFiscale }



{-------------------------------------------------------------------------------
  Italian personal fiscal code test.
}
procedure TMainForm.CFtestBtnClick(Sender: TObject);
var S: string;
begin
  {- first strip all blanks to ensure a correct string, the use of
     a dummy string is only for clearness }
  S := StripAllBlanks(CodiceFiscale.Text);
  if CheckCodiceFiscale(S) then
     OutCodFis.Caption := 'Ok!   '+S
  else OutCodFis.Caption := '<ERR> '+S;
end; {- CFtestBtnClick }



{-------------------------------------------------------------------------------
  Checking if the italian businness fiscal code is correct.
  The last digit is the checksum.
  The function is stand-alone so you can use with no problems
}
function TMainForm.CheckPartitaIva(PI: string): boolean;
var Check, Count1, It, xC: integer;
begin
  It := 1;  Count1 := 0;
  repeat
    val(PI[It], Check, xC);
    Count1 := Count1 + Check;
    val(PI[It+1], Check, xC);
    Count1 := Count1 + Trunc(int((Check*2)/10)) + ((Check*2) mod 10);
    It := It + 2;
  until It > 9;

  val(PI[11], Check, xC);
  if Check = (10 - (Count1 Mod 10)) then
        CheckPartitaIva := True
  else CheckPartitaIva := False;
end; {- CheckPartitaIva }



{-------------------------------------------------------------------------------
  Italian businness fiscal code test.
}
procedure TMainForm.PItestBtnClick(Sender: TObject);
var S: string;
begin
  {- first strip all blanks to ensure a correct string, the use of
     a dummy string is only for clearness }
  S := StripAllBlanks(PartitaIva.Text);
  if CheckPartitaIva(S) then
     OutPiva.Caption := 'Ok!   '+S
  else OutPiva.Caption := '<ERR> '+S;
end; {- PItestBtnClick }


{-------------------------------------------------------------------------------
}
procedure TMainForm.CloseBtnClick(Sender: TObject);
begin
  Close;
end;


{-------------------------------------------------------------------------------
}
procedure TMainForm.ReadIniFile;
var
  MyIni: TIniFile;
  IniName: string;
begin
  IniName := ExtractFilePath(Application.ExeName)+'CodFis.ini';
  MyIni := TIniFile.Create(IniName);
  try
    Rate := MyIni.ReadFloat('GENERAL', 'Rate', 1700.00);
  finally
    MyIni.Free;
  end;
  Edit_Rate.Text := CurrToStrF(Rate, ffNumber	, 2);
  Edit_Lira.Text := CurrToStrF(0.0, ffNumber	, 0);
  Edit_Euro.Text := CurrToStrF(0.0, ffNumber	, 2);
end;


{-------------------------------------------------------------------------------
}
procedure TMainForm.EuroCalc;
begin
  Lira   := StrToCurr(Edit_Lira.Text);
  Euro   := StrToCurr(Edit_Euro.Text);
  Rate   := StrToCurr(Edit_Rate.Text);
  if EuroFlag.Checked then
     begin
       Euro := Lira / Rate;
     end else
     begin
       Lira := Euro * Rate;
     end;
  CalcLira.Caption := CurrToStrF(Lira, ffNumber	, 0);
  CalcEuro.Caption := CurrToStrF(Euro, ffNumber	, 2);
end;


{-------------------------------------------------------------------------------
}
procedure TMainForm.ExecBtnClick(Sender: TObject);
begin
  EuroCalc;
end;

end.
